@@ -3,7 +3,7 @@
  * Copyright (c) 2004-2005 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
- * Copyright (c) 2004-2005 The University of Tennessee and The University
+ * Copyright (c) 2004-2017 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
  * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart,
@@ -11,9 +11,11 @@
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
  * Copyright (c) 2008-2014 University of Houston. All rights reserved.
- * Copyright (c) 2015 Cisco Systems, Inc.  All rights reserved.
+ * Copyright (c) 2015      Cisco Systems, Inc.  All rights reserved.
  * Copyright (c) 2015      Los Alamos National Security, LLC. All rights
  *                         reserved.
+ * Copyright (c) 2017-2018 Research Organization for Information Science
+ *                         and Technology (RIST). All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -25,8 +27,9 @@
 #include "fcoll_two_phase.h"
 #include "mpi.h"
 #include "ompi/constants.h"
+#include "ompi/communicator/communicator.h"
 #include "ompi/mca/fcoll/fcoll.h"
-#include "ompi/mca/io/ompio/io_ompio.h"
+#include "ompi/mca/common/ompio/common_ompio.h"
 #include "ompi/mca/io/io.h"
 #include "opal/mca/base/base.h"
 #include "math.h"
@@ -51,10 +54,10 @@ typedef struct flat_list_node {
 }Flatlist_node;
 
 /* local function declarations  */
-static int two_phase_read_and_exch(mca_io_ompio_file_t *fh,
+static int two_phase_read_and_exch(ompio_file_t *fh,
 				   void *buf,
 				   MPI_Datatype datatype,
-				   mca_io_ompio_access_array_t *others_req,
+				   mca_common_ompio_access_array_t *others_req,
 				   struct iovec *offset_len,
 				   int contig_access_count,
 				   OMPI_MPI_OFFSET_TYPE min_st_offset,
@@ -65,7 +68,7 @@ static int two_phase_read_and_exch(mca_io_ompio_file_t *fh,
 				   size_t *buf_idx, int striping_unit,
 				   int num_io_procs, int *aggregator_list);
 
-static int  two_phase_exchange_data(mca_io_ompio_file_t *fh,
+static int  two_phase_exchange_data(ompio_file_t *fh,
 				    void *buf,
 				    struct iovec *offset_length,
 				    int *send_size, int *start_pos,
@@ -78,14 +81,14 @@ static int  two_phase_exchange_data(mca_io_ompio_file_t *fh,
 				    OMPI_MPI_OFFSET_TYPE *fd_start,
 				    OMPI_MPI_OFFSET_TYPE *fd_end,
 				    Flatlist_node *flat_buf,
-				    mca_io_ompio_access_array_t *others_req,
+				    mca_common_ompio_access_array_t *others_req,
 				    int iter,
 				    size_t *buf_idx, MPI_Aint buftype_extent,
 				    int striping_unit, int num_io_procs,
 				    int *aggregator_list);
 
 
-static void two_phase_fill_user_buffer(mca_io_ompio_file_t *fh,
+static void two_phase_fill_user_buffer(ompio_file_t *fh,
 				       void *buf,
 				       Flatlist_node *flat_buf,
 				       char **recv_buf,
@@ -116,7 +119,7 @@ double read_exch = 0.0, start_rexch = 0.0, end_rexch = 0.0;
 
 
 int
-mca_fcoll_two_phase_file_read_all (mca_io_ompio_file_t *fh,
+mca_fcoll_two_phase_file_read_all (ompio_file_t *fh,
 				   void *buf,
 				   int count,
 				   struct ompi_datatype_t *datatype,
@@ -138,22 +141,23 @@ mca_fcoll_two_phase_file_read_all (mca_io_ompio_file_t *fh,
     OMPI_MPI_OFFSET_TYPE *start_offsets=NULL, *end_offsets=NULL;
     OMPI_MPI_OFFSET_TYPE *fd_start=NULL, *fd_end=NULL, min_st_offset = 0;
     Flatlist_node *flat_buf=NULL;
-    mca_io_ompio_access_array_t *my_req=NULL, *others_req=NULL;
+    mca_common_ompio_access_array_t *my_req=NULL, *others_req=NULL;
 #if OMPIO_FCOLL_WANT_TIME_BREAKDOWN
-    mca_io_ompio_print_entry nentry;
+    mca_common_ompio_print_entry nentry;
 #endif
 //    if (opal_datatype_is_predefined(&datatype->super)) {
 //	fh->f_flags = fh->f_flags |  OMPIO_CONTIGUOUS_MEMORY;
 //    }
 
     if (! (fh->f_flags & OMPIO_CONTIGUOUS_MEMORY)) {
-	ret =   fh->f_decode_datatype ((struct mca_io_ompio_file_t *)fh,
-				       datatype,
-				       count,
-				       buf,
-				       &max_data,
-				       &temp_iov,
-				       &iov_count);
+	ret =   mca_common_ompio_decode_datatype ((struct ompio_file_t *)fh,
+				                  datatype,
+				                  count,
+				                  buf,
+				                  &max_data,
+                                                  fh->f_mem_convertor,
+				                  &temp_iov,
+				                  &iov_count);
 	if (OMPI_SUCCESS != ret ){
 	    goto exit;
 	}
@@ -165,11 +169,11 @@ mca_fcoll_two_phase_file_read_all (mca_io_ompio_file_t *fh,
 	for (ti = 0; ti < iov_count; ti++){
 
 	    decoded_iov[ti].iov_base = (IOVBASE_TYPE *)
-		((OPAL_PTRDIFF_TYPE)temp_iov[ti].iov_base - recv_buf_addr);
+		((ptrdiff_t)temp_iov[ti].iov_base - recv_buf_addr);
 	    decoded_iov[ti].iov_len = temp_iov[ti].iov_len;
 #if DEBUG
 	    printf("d_offset[%d]: %ld, d_len[%d]: %ld\n",
-		   ti, (OPAL_PTRDIFF_TYPE)decoded_iov[ti].iov_base,
+		   ti, (ptrdiff_t)decoded_iov[ti].iov_base,
 		   ti, decoded_iov[ti].iov_len);
 #endif
 	}
@@ -183,21 +187,25 @@ mca_fcoll_two_phase_file_read_all (mca_io_ompio_file_t *fh,
 	status->_ucount = max_data;
     }
 
-    fh->f_get_num_aggregators (&two_phase_num_io_procs);
+    two_phase_num_io_procs = fh->f_get_mca_parameter_value ( "num_aggregators", strlen ("num_aggregators"));
+    if ( OMPI_ERR_MAX == two_phase_num_io_procs ) {
+        ret = OMPI_ERROR;
+        goto exit;
+    }
     if (-1 == two_phase_num_io_procs ){
-	ret = fh->f_set_aggregator_props ((struct mca_io_ompio_file_t *)fh,
-					  two_phase_num_io_procs,
-					  max_data);
+	ret = mca_common_ompio_set_aggregator_props ((struct ompio_file_t *)fh,
+					             two_phase_num_io_procs,
+					             max_data);
 	if (OMPI_SUCCESS != ret){
             goto exit;
 	}
 
-	two_phase_num_io_procs = fh->f_final_num_aggrs;
+	two_phase_num_io_procs = fh->f_num_aggrs;
 
     }
 
     if (two_phase_num_io_procs > fh->f_size){
-	two_phase_num_io_procs = fh->f_size;
+        two_phase_num_io_procs = fh->f_size;
     }
 
     aggregator_list = (int *) calloc (two_phase_num_io_procs, sizeof(int));
@@ -206,11 +214,18 @@ mca_fcoll_two_phase_file_read_all (mca_io_ompio_file_t *fh,
 	goto exit;
     }
 
-    for (i=0; i< two_phase_num_io_procs; i++){
-	aggregator_list[i] = i * fh->f_size / two_phase_num_io_procs;
+    if ( OMPI_COMM_IS_MAPBY_NODE (&ompi_mpi_comm_world.comm) ) {
+        for (i =0; i< two_phase_num_io_procs; i++){
+            aggregator_list[i] = i;
+        }
     }
+    else {
+        for (i =0; i< two_phase_num_io_procs; i++){
+            aggregator_list[i] = i * fh->f_size / two_phase_num_io_procs;
+        }
+    }        
 
-    ret = fh->f_generate_current_file_view ((struct mca_io_ompio_file_t *)fh,
+    ret = fh->f_generate_current_file_view ((struct ompio_file_t *)fh,
 					    max_data,
 					    &iov,
 					    &local_count);
@@ -220,13 +235,13 @@ mca_fcoll_two_phase_file_read_all (mca_io_ompio_file_t *fh,
     }
 
     long_max_data = (long) max_data;
-    ret = fh->f_comm->c_coll.coll_allreduce (&long_max_data,
+    ret = fh->f_comm->c_coll->coll_allreduce (&long_max_data,
 					     &long_total_bytes,
 					     1,
 					     MPI_LONG,
 					     MPI_SUM,
 					     fh->f_comm,
-					     fh->f_comm->c_coll.coll_allreduce_module);
+					     fh->f_comm->c_coll->coll_allreduce_module);
 
     if ( OMPI_SUCCESS != ret ) {
 	goto exit;
@@ -334,27 +349,27 @@ mca_fcoll_two_phase_file_read_all (mca_io_ompio_file_t *fh,
 	goto exit;
     }
 
-    ret = fh->f_comm->c_coll.coll_allgather(&start_offset,
+    ret = fh->f_comm->c_coll->coll_allgather(&start_offset,
 					    1,
 					    OMPI_OFFSET_DATATYPE,
 					    start_offsets,
 					    1,
 					    OMPI_OFFSET_DATATYPE,
 					    fh->f_comm,
-					    fh->f_comm->c_coll.coll_allgather_module);
+					    fh->f_comm->c_coll->coll_allgather_module);
 
     if ( OMPI_SUCCESS != ret ){
 	goto exit;
     }
 
-    ret = fh->f_comm->c_coll.coll_allgather(&end_offset,
+    ret = fh->f_comm->c_coll->coll_allgather(&end_offset,
 					    1,
 					    OMPI_OFFSET_DATATYPE,
 					    end_offsets,
 					    1,
 					    OMPI_OFFSET_DATATYPE,
 					    fh->f_comm,
-					    fh->f_comm->c_coll.coll_allgather_module);
+					    fh->f_comm->c_coll->coll_allgather_module);
 
 
     if ( OMPI_SUCCESS != ret ){
@@ -479,9 +494,9 @@ mca_fcoll_two_phase_file_read_all (mca_io_ompio_file_t *fh,
     nentry.nprocs_for_coll = two_phase_num_io_procs;
 
 
-    if (!fh->f_full_print_queue(READ_PRINT_QUEUE)){
-	fh->f_register_print_entry(READ_PRINT_QUEUE,
-				   nentry);
+    if (!mca_common_ompio_full_print_queue(fh->f_coll_read_time)){
+	mca_common_ompio_register_print_entry(fh->f_coll_read_time,
+                                              nentry);
     }
 #endif
 
@@ -514,10 +529,10 @@ exit:
 
 
 
-static int two_phase_read_and_exch(mca_io_ompio_file_t *fh,
+static int two_phase_read_and_exch(ompio_file_t *fh,
 				   void *buf,
 				   MPI_Datatype datatype,
-				   mca_io_ompio_access_array_t *others_req,
+				   mca_common_ompio_access_array_t *others_req,
 				   struct iovec *offset_len,
 				   int contig_access_count,
 				   OMPI_MPI_OFFSET_TYPE min_st_offset,
@@ -565,7 +580,7 @@ static int two_phase_read_and_exch(mca_io_ompio_file_t *fh,
 	}
     }
 
-    fh->f_get_bytes_per_agg ( &two_phase_cycle_buffer_size);
+    two_phase_cycle_buffer_size = fh->f_bytes_per_agg;
     ntimes = (int)((end_loc - st_loc + two_phase_cycle_buffer_size)/
 		   two_phase_cycle_buffer_size);
 
@@ -573,13 +588,13 @@ static int two_phase_read_and_exch(mca_io_ompio_file_t *fh,
 	ntimes = 0;
     }
 
-    fh->f_comm->c_coll.coll_allreduce (&ntimes,
+    fh->f_comm->c_coll->coll_allreduce (&ntimes,
 				       &max_ntimes,
 				       1,
 				       MPI_INT,
 				       MPI_MAX,
 				       fh->f_comm,
-				       fh->f_comm->c_coll.coll_allreduce_module);
+				       fh->f_comm->c_coll->coll_allreduce_module);
 
     if (ntimes){
 	read_buf = (char *) calloc (two_phase_cycle_buffer_size, sizeof(char));
@@ -671,7 +686,7 @@ static int two_phase_read_and_exch(mca_io_ompio_file_t *fh,
 		    }
 		    if (req_off < real_off + real_size) {
 			count[i]++;
-			PMPI_Address(read_buf+req_off-real_off,
+			PMPI_Get_address(read_buf+req_off-real_off,
 				     &(others_req[i].mem_ptrs[j]));
 
 			send_size[i] += (int)(OMPIO_MIN(real_off + real_size - req_off,
@@ -708,8 +723,8 @@ static int two_phase_read_and_exch(mca_io_ompio_file_t *fh,
 #endif
 
 	    len = size * byte_size;
-	    fh->f_io_array = (mca_io_ompio_io_array_t *)calloc
-		(1,sizeof(mca_io_ompio_io_array_t));
+	    fh->f_io_array = (mca_common_ompio_io_array_t *)calloc
+		(1,sizeof(mca_common_ompio_io_array_t));
 	    if (NULL == fh->f_io_array) {
 		opal_output(1, "OUT OF MEMORY\n");
                 ret = OMPI_ERR_OUT_OF_RESOURCE;
@@ -809,7 +824,7 @@ exit:
 
 }
 
-static int two_phase_exchange_data(mca_io_ompio_file_t *fh,
+static int two_phase_exchange_data(ompio_file_t *fh,
 				   void *buf, struct iovec *offset_len,
 				   int *send_size, int *start_pos, int *recv_size,
 				   int *count, int *partial_send,
@@ -819,7 +834,7 @@ static int two_phase_exchange_data(mca_io_ompio_file_t *fh,
 				   OMPI_MPI_OFFSET_TYPE *fd_start,
 				   OMPI_MPI_OFFSET_TYPE *fd_end,
 				   Flatlist_node *flat_buf,
-				   mca_io_ompio_access_array_t *others_req,
+				   mca_common_ompio_access_array_t *others_req,
 				   int iter, size_t *buf_idx,
 				   MPI_Aint buftype_extent, int striping_unit,
 				   int two_phase_num_io_procs, int *aggregator_list)
@@ -837,14 +852,14 @@ static int two_phase_exchange_data(mca_io_ompio_file_t *fh,
     start_rcomm_time = MPI_Wtime();
 #endif
 
-    ret = fh->f_comm->c_coll.coll_alltoall (send_size,
+    ret = fh->f_comm->c_coll->coll_alltoall (send_size,
 					    1,
 					    MPI_INT,
 					    recv_size,
 					    1,
 					    MPI_INT,
 					    fh->f_comm,
-					    fh->f_comm->c_coll.coll_alltoall_module);
+					    fh->f_comm->c_coll->coll_alltoall_module);
 
     if ( OMPI_SUCCESS != ret ){
 	goto exit;
@@ -1047,7 +1062,7 @@ exit:
 
 
 
-static void two_phase_fill_user_buffer(mca_io_ompio_file_t *fh,
+static void two_phase_fill_user_buffer(ompio_file_t *fh,
 				       void *buf,
 				       Flatlist_node *flat_buf,
 				       char **recv_buf,
