@@ -1,23 +1,27 @@
+/* -*- Mode: C; c-basic-offset:4 ; indent-tabs-mode:nil -*- */
 /*
  * Copyright (c) 2004-2005 The Trustees of Indiana University.
  *                         All rights reserved.
  * Copyright (c) 2004-2005 The Trustees of the University of Tennessee.
  *                         All rights reserved.
- * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart, 
+ * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart,
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
+ * Copyright (c) 2014      Los Alamos National Security, LLC. All rights
+ *                         reserved.
+ * Copyright (c) 2016-2017 IBM Corporation. All rights reserved.
  * $COPYRIGHT$
- * 
+ *
  * Additional copyrights may follow
- * 
+ *
  * $HEADER$
  */
 
 #include "ompi_config.h"
 
 #include "ompi/constants.h"
-#include "opal/mca/mca.h"
+#include "ompi/mca/mca.h"
 #include "opal/mca/base/base.h"
 #include "ompi/mca/osc/osc.h"
 #include "ompi/mca/osc/base/base.h"
@@ -31,7 +35,7 @@ ompi_osc_base_select(ompi_win_t *win,
                      size_t size,
                      int disp_unit,
                      ompi_communicator_t *comm,
-                     ompi_info_t *info,
+                     opal_info_t *info,
                      int flavor,
                      int *model)
 {
@@ -51,7 +55,14 @@ ompi_osc_base_select(ompi_win_t *win,
             ((mca_base_component_list_item_t*) item)->cli_component;
 
         priority = component->osc_query(win, base, size, disp_unit, comm, info, flavor);
-        if (priority < 0) continue;
+        if (priority < 0) {
+            if (MPI_WIN_FLAVOR_SHARED == flavor && OMPI_ERR_RMA_SHARED == priority) {
+                /* NTH: quick fix to return OMPI_ERR_RMA_SHARED */
+                return OMPI_ERR_RMA_SHARED;
+            }
+            continue;
+        }
+
         if (priority > best_priority) {
             best_component = component;
             best_priority = priority;
