@@ -1,3 +1,4 @@
+/* -*- Mode: C; c-basic-offset:4 ; indent-tabs-mode:nil -*- */
 /*
  * Copyright (c) 2004-2007 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
@@ -5,21 +6,24 @@
  * Copyright (c) 2004-2005 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
- * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart, 
+ * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart,
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2006 The Regents of the University of California.
  *                         All rights reserved.
- * Copyright (c) 2006-2007 Los Alamos National Security, LLC.  All rights
- *                         reserved. 
+ * Copyright (c) 2006-2015 Los Alamos National Security, LLC.  All rights
+ *                         reserved.
+ * Copyright (c) 2011      Sandia National Laboratories. All rights reserved.
+ * Copyright (c) 2015      Research Organization for Information Science
+ *                         and Technology (RIST). All rights reserved.
  * $COPYRIGHT$
- * 
+ *
  * Additional copyrights may follow
- * 
+ *
  * $HEADER$
  */
 /**
  * @file
- * 
+ *
  * P2P Management Layer (PML)
  *
  * An MCA component type that provides the P2P interface functionality
@@ -47,7 +51,7 @@
  * make downcalls into the PML to provide the initial list of
  * processes (ompi_proc_t instances), and notification of changes
  * (add/delete).
- * 
+ *
  * The PML module must select the set of BTL components that are to be
  * used to reach a given destination. These should be cached on a PML
  * specific data structure that is hung off the ompi_proc_t.
@@ -57,73 +61,28 @@
  * over the available BTLs.
  *
  */
-                                                                                         
+
 #ifndef MCA_PML_H
 #define MCA_PML_H
 
 #include "ompi_config.h"
-#include "opal/class/opal_list.h"
-#include "opal/mca/mca.h"
+#include "ompi/mca/mca.h"
 #include "mpi.h" /* needed for MPI_ANY_TAG */
+#include "ompi/mca/pml/pml_constants.h"
+#include "ompi/request/request.h"
 
-#if defined(c_plusplus) || defined(__cplusplus)
-extern "C" {
-#endif
+BEGIN_C_DECLS
 
 /*
  * PML component types
  */
 
 typedef uint64_t mca_pml_sequence_t;
-
-/**
- * Base PML proc structure
- *
- * Base PML structure for storing proc information.
- * Note that the mca_pml_proc_t structure can not be instantiated
- * directly, so each PML *must* provide a class that inherits from
- * this class and provides the necessary integration logic.
- */
-struct mca_pml_proc_t {
-    opal_list_item_t super;
-    struct ompi_proc_t *proc_ompi;    /**< back-pointer to ompi_proc_t */
-    opal_mutex_t proc_lock;           /**< lock to protect against concurrent access */
-};
-typedef struct mca_pml_proc_t mca_pml_proc_t; 
-
-
-/** 
- * Base PML endpoint structure
- *
- * Base PML structure for caching endpoint information on a proc.  A
- * pointer to an mca_pml_endpoint_t is maintained on each ompi_proc_t,
- * in the proc_pml field, to provide per-process cache information.
- * The data is opaque to the active PML -- no other subsystem will
- * attempt to access the information in the cache.
- *
- * The PML is responsible for allocation and deallocation of the
- * endpoint data during pml_add_procs and pml_del_procs.
- */
-struct mca_pml_endpoint_t;
-
-
-typedef enum {
-    MCA_PML_BASE_SEND_SYNCHRONOUS,
-    MCA_PML_BASE_SEND_COMPLETE,
-    MCA_PML_BASE_SEND_BUFFERED,
-    MCA_PML_BASE_SEND_READY,
-    MCA_PML_BASE_SEND_STANDARD,
-    MCA_PML_BASE_SEND_SIZE
-} mca_pml_base_send_mode_t;
-
-
-#define OMPI_ANY_TAG    MPI_ANY_TAG
-#define OMPI_ANY_SOURCE MPI_ANY_SOURCE
-#define OMPI_PROC_NULL  MPI_PROC_NULL
+struct ompi_proc_t;
 
 /**
  * MCA->PML Called by MCA framework to initialize the component.
- * 
+ *
  * @param priority (OUT) Relative priority or ranking used by MCA to
  * selected a component.
  *
@@ -135,8 +94,8 @@ typedef enum {
  * indicates whether multiple threads may invoke this component
  * simultaneously or not.
  */
-typedef struct mca_pml_base_module_1_0_0_t * (*mca_pml_base_component_init_fn_t)(
-    int *priority, 
+typedef struct mca_pml_base_module_1_0_1_t * (*mca_pml_base_component_init_fn_t)(
+    int *priority,
     bool enable_progress_threads,
     bool enable_mpi_threads);
 
@@ -182,7 +141,7 @@ typedef int (*mca_pml_base_module_add_procs_fn_t)(struct ompi_proc_t **procs, si
  * @param  nprocs  Size of process array
  * @return         OMPI_SUCCESS or failure status.
  *
- * Provides a notification to the PML that processes have 
+ * Provides a notification to the PML that processes have
  * gone away, and provides the PML the opportunity to cleanup
  * any data cached on the ompi_proc_t data structure.
  */
@@ -203,8 +162,8 @@ typedef int (*mca_pml_base_module_enable_fn_t)(
  * For non-threaded case, provides MCA the opportunity to
  * progress outstanding requests on all btls.
  *
- *  * @return        Count of "completions", a metric of 
- *                   how many items where completed in the call 
+ *  * @return        Count of "completions", a metric of
+ *                   how many items where completed in the call
  *                   to progress.
 */
 typedef int (*mca_pml_base_module_progress_fn_t)(void);
@@ -238,7 +197,7 @@ typedef int (*mca_pml_base_module_add_comm_fn_t)(struct ompi_communicator_t* com
 typedef int (*mca_pml_base_module_del_comm_fn_t)(struct ompi_communicator_t* comm);
 
 /**
- *  Initialize a persistent receive request. 
+ *  Initialize a persistent receive request.
  *
  *  @param buf (IN)         User buffer.
  *  @param count (IN)       Number of elements of the specified datatype.
@@ -250,17 +209,17 @@ typedef int (*mca_pml_base_module_del_comm_fn_t)(struct ompi_communicator_t* com
  *  @return                 OMPI_SUCCESS or failure status.
  */
 typedef int (*mca_pml_base_module_irecv_init_fn_t)(
-    void *buf,                           
-    size_t count,                         
-    struct ompi_datatype_t *datatype,              
+    void *buf,
+    size_t count,
+    struct ompi_datatype_t *datatype,
     int src,
-    int tag,                                
+    int tag,
     struct ompi_communicator_t* comm,
-    struct ompi_request_t **request           
+    struct ompi_request_t **request
 );
 
 /**
- *  Post a receive request. 
+ *  Post a receive request.
  *
  *  @param buf (IN)         User buffer.
  *  @param count (IN)       Number of elements of the specified datatype.
@@ -280,9 +239,16 @@ typedef int (*mca_pml_base_module_irecv_fn_t)(
     struct ompi_communicator_t* comm,
     struct ompi_request_t **request
 );
+typedef int (*mca_pml_base_module_imrecv_fn_t)(
+    void *buf,
+    size_t count,
+    struct ompi_datatype_t *datatype,
+    struct ompi_message_t **message,
+    struct ompi_request_t **request
+);
 
 /**
- *  Post a receive and wait for completion. 
+ *  Post a receive and wait for completion.
  *
  *  @param buf (IN)         User buffer
  *  @param count (IN)       Number of elements of the specified datatype
@@ -302,9 +268,16 @@ typedef int (*mca_pml_base_module_recv_fn_t)(
     struct ompi_communicator_t* comm,
     ompi_status_public_t* status
 );
+typedef int (*mca_pml_base_module_mrecv_fn_t)(
+    void *buf,
+    size_t count,
+    struct ompi_datatype_t *datatype,
+    struct ompi_message_t **message,
+    ompi_status_public_t* status
+);
 
 /**
- *  Initialize a persistent send request. 
+ *  Initialize a persistent send request.
  *
  *  @param buf (IN)         User buffer.
  *  @param count (IN)       Number of elements of the specified datatype.
@@ -317,7 +290,7 @@ typedef int (*mca_pml_base_module_recv_fn_t)(
  *  @return                 OMPI_SUCCESS or failure status.
  */
 typedef int (*mca_pml_base_module_isend_init_fn_t)(
-    void *buf,
+    const void *buf,
     size_t count,
     struct ompi_datatype_t *datatype,
     int dst,
@@ -329,7 +302,7 @@ typedef int (*mca_pml_base_module_isend_init_fn_t)(
 
 
 /**
- *  Post a send request. 
+ *  Post a send request.
  *
  *  @param buf (IN)         User buffer.
  *  @param count (IN)       Number of elements of the specified datatype.
@@ -342,7 +315,7 @@ typedef int (*mca_pml_base_module_isend_init_fn_t)(
  *  @return                 OMPI_SUCCESS or failure status.
  */
 typedef int (*mca_pml_base_module_isend_fn_t)(
-    void *buf,
+    const void *buf,
     size_t count,
     struct ompi_datatype_t *datatype,
     int dst,
@@ -366,7 +339,7 @@ typedef int (*mca_pml_base_module_isend_fn_t)(
  *  @return                 OMPI_SUCCESS or failure status.
  */
 typedef int (*mca_pml_base_module_send_fn_t)(
-    void *buf,
+    const void *buf,
     size_t count,
     struct ompi_datatype_t *datatype,
     int dst,
@@ -378,14 +351,11 @@ typedef int (*mca_pml_base_module_send_fn_t)(
 /**
  * Initiate one or more persistent requests.
  *
- * @param count    Number of requests
- * @param request  Array of persistent requests
- * @return         OMPI_SUCCESS or failure status.
+ * @param count (IN)        Number of requests
+ * @param requests (IN/OUT) Array of persistent requests
+ * @return                  OMPI_SUCCESS or failure status.
  */
-typedef int (*mca_pml_base_module_start_fn_t)(
-    size_t count,
-    struct ompi_request_t** requests
-);
+typedef ompi_request_start_fn_t mca_pml_base_module_start_fn_t;
 
 /**
  * Probe to poll for pending recv.
@@ -406,6 +376,15 @@ typedef int (*mca_pml_base_module_iprobe_fn_t)(
     ompi_status_public_t *status
 );
 
+typedef int (*mca_pml_base_module_improbe_fn_t)(
+    int src,
+    int tag,
+    struct ompi_communicator_t* comm,
+    int *matched,
+    struct ompi_message_t **message,
+    ompi_status_public_t *status
+);
+
 /**
  * Blocking probe to wait for pending recv.
  *
@@ -423,9 +402,17 @@ typedef int (*mca_pml_base_module_probe_fn_t)(
     ompi_status_public_t *status
 );
 
+typedef int (*mca_pml_base_module_mprobe_fn_t)(
+    int src,
+    int tag,
+    struct ompi_communicator_t* comm,
+    struct ompi_message_t **message,
+    ompi_status_public_t *status
+);
+
 /**
  * Cancel pending operation.
- * 
+ *
  * @param request (IN)    Request
  * @return                OMPI_SUCCESS or failure status.
  *
@@ -437,7 +424,7 @@ typedef int (*mca_pml_base_module_cancel_fn_t)(
 
 /**
  * Has a request been cancelled?
- * 
+ *
  * @param request (IN)    Request
  * @return                OMPI_SUCCESS or failure status.
  *
@@ -490,13 +477,18 @@ typedef int (*mca_pml_base_module_dump_fn_t)(
  */
 typedef int (*mca_pml_base_module_ft_event_fn_t) (int status);
 
-
+/**
+ * pml module flags
+ */
+/** PML requires requires all procs in the job on the first call to
+ * add_procs */
+#define MCA_PML_BASE_FLAG_REQUIRE_WORLD 0x00000001
 
 /**
  *  PML instance.
  */
 
-struct mca_pml_base_module_1_0_0_t {
+struct mca_pml_base_module_1_0_1_t {
 
     /* downcalls from MCA to PML */
     mca_pml_base_module_add_procs_fn_t    pml_add_procs;
@@ -516,6 +508,10 @@ struct mca_pml_base_module_1_0_0_t {
     mca_pml_base_module_iprobe_fn_t       pml_iprobe;
     mca_pml_base_module_probe_fn_t        pml_probe;
     mca_pml_base_module_start_fn_t        pml_start;
+    mca_pml_base_module_improbe_fn_t      pml_improbe;
+    mca_pml_base_module_mprobe_fn_t       pml_mprobe;
+    mca_pml_base_module_imrecv_fn_t       pml_imrecv;
+    mca_pml_base_module_mrecv_fn_t        pml_mrecv;
 
     /* diagnostics */
     mca_pml_base_module_dump_fn_t         pml_dump;
@@ -524,29 +520,29 @@ struct mca_pml_base_module_1_0_0_t {
     mca_pml_base_module_ft_event_fn_t     pml_ft_event;
 
     /* maximum constant sizes */
-    int                                   pml_max_contextid;
+    uint32_t                              pml_max_contextid;
     int                                   pml_max_tag;
+    int                                   pml_flags;
 };
-typedef struct mca_pml_base_module_1_0_0_t mca_pml_base_module_1_0_0_t;
-typedef mca_pml_base_module_1_0_0_t mca_pml_base_module_t;
+typedef struct mca_pml_base_module_1_0_1_t mca_pml_base_module_1_0_1_t;
+typedef mca_pml_base_module_1_0_1_t mca_pml_base_module_t;
 
 /*
  * Macro for use in components that are of type pml
  */
 #define MCA_PML_BASE_VERSION_2_0_0 \
-  MCA_BASE_VERSION_2_0_0, \
-  "pml", 2, 0, 0
+    OMPI_MCA_BASE_VERSION_2_1_0("pml", 2, 0, 0)
 
     /*
      * macro for doing direct call / call through struct
      */
-#if MCA_pml_DIRECT_CALL
+#if MCA_ompi_pml_DIRECT_CALL
 
-#include MCA_pml_DIRECT_CALL_HEADER
+#include MCA_ompi_pml_DIRECT_CALL_HEADER
 
 #define MCA_PML_CALL_STAMP(a, b) mca_pml_ ## a ## _ ## b
 #define MCA_PML_CALL_EXPANDER(a, b) MCA_PML_CALL_STAMP(a,b)
-#define MCA_PML_CALL(a) MCA_PML_CALL_EXPANDER(MCA_pml_DIRECT_CALL_COMPONENT, a)
+#define MCA_PML_CALL(a) MCA_PML_CALL_EXPANDER(MCA_ompi_pml_DIRECT_CALL_COMPONENT, a)
 
 #else
 #define MCA_PML_CALL(a) mca_pml.pml_ ## a
@@ -554,8 +550,10 @@ typedef mca_pml_base_module_1_0_0_t mca_pml_base_module_t;
 
 OMPI_DECLSPEC extern mca_pml_base_module_t mca_pml;
 
-
-#if defined(c_plusplus) || defined(__cplusplus)
+static inline bool mca_pml_base_requires_world (void)
+{
+    return !!(mca_pml.pml_flags & MCA_PML_BASE_FLAG_REQUIRE_WORLD);
 }
-#endif
+
+END_C_DECLS
 #endif /* MCA_PML_H */
