@@ -2,7 +2,7 @@
  * Copyright (c) 2004-2007 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
- * Copyright (c) 2004-2006 The University of Tennessee and The University
+ * Copyright (c) 2004-2018 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
  * Copyright (c) 2004-2008 High Performance Computing Center Stuttgart,
@@ -30,6 +30,7 @@
 #include "ompi/errhandler/errhandler.h"
 #include "ompi/request/request.h"
 #include "ompi/memchecker.h"
+#include "ompi/runtime/ompi_spc.h"
 
 #if OMPI_BUILD_MPI_PROFILING
 #if OPAL_HAVE_WEAK_SYMBOLS
@@ -45,6 +46,8 @@ int MPI_Testsome(int incount, MPI_Request requests[],
                  int *outcount, int indices[],
                  MPI_Status statuses[])
 {
+    SPC_RECORD(OMPI_SPC_TESTSOME, 1);
+
     MEMCHECKER(
         int j;
         for (j = 0; j < incount; j++){
@@ -77,16 +80,20 @@ int MPI_Testsome(int incount, MPI_Request requests[],
         return OMPI_SUCCESS;
     }
 
+    OPAL_CR_ENTER_LIBRARY();
 
     if (OMPI_SUCCESS == ompi_request_test_some(incount, requests, outcount,
                                                indices, statuses)) {
+        OPAL_CR_EXIT_LIBRARY();
         return MPI_SUCCESS;
     }
 
     if (MPI_SUCCESS !=
         ompi_errhandler_request_invoke(incount, requests, FUNC_NAME)) {
+        OPAL_CR_EXIT_LIBRARY();
         return MPI_ERR_IN_STATUS;
     }
 
+    OPAL_CR_EXIT_LIBRARY();
     return MPI_SUCCESS;
 }
