@@ -2,6 +2,11 @@
 /*
  * Copyright (c) 2012-2013 Los Alamos National Security, LLC. All rights
  *                         reserved.
+ * Copyright (c) 2014 Cisco Systems, Inc.  All rights reserved.
+ * Copyright (c) 2017      IBM Corporation. All rights reserved.
+ * Copyright (c) 2020      The University of Tennessee and The University
+ *                         of Tennessee Research Foundation.  All rights
+ *                         reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -19,7 +24,6 @@
 #include "ompi/mpi/tool/profile/defines.h"
 #endif
 
-static const char FUNC_NAME[] = "MPI_T_category_get_num";
 
 int MPI_T_category_get_num (int *num_cat)
 {
@@ -28,12 +32,12 @@ int MPI_T_category_get_num (int *num_cat)
     }
 
     if (MPI_PARAM_CHECK && NULL == num_cat) {
-        return MPI_ERR_ARG;
+        return MPI_T_ERR_INVALID;
     }
 
-    mpit_lock ();
+    ompi_mpit_lock ();
     *num_cat = mca_base_var_group_get_count ();
-    mpit_unlock ();
+    ompi_mpit_unlock ();
 
     return MPI_SUCCESS;
 }
