@@ -10,9 +10,10 @@
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
  * Copyright (c) 2007-2011 Cisco Systems, Inc.  All rights reserved.
- * Copyright (c) 2013-2017 Intel, Inc. All rights reserved.
+ * Copyright (c) 2013-2018 Intel, Inc.  All rights reserved.
  * Copyright (c) 2016      Research Organization for Information Science
  *                         and Technology (RIST). All rights reserved.
+ * Copyright (c) 2021      IBM Corporation.  All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -40,7 +41,6 @@
 #include "orte/util/proc_info.h"
 #include "orte/runtime/orte_globals.h"
 #include "orte/mca/errmgr/errmgr.h"
-#include "orte/util/regex.h"
 
 #include "orte/mca/ess/ess.h"
 #include "orte/mca/ess/base/base.h"
@@ -54,7 +54,7 @@ static int rte_finalize(void);
 orte_ess_base_module_t orte_ess_lsf_module = {
     rte_init,
     rte_finalize,
-    orte_ess_base_app_abort,
+    NULL,
     NULL /* ft_event */
 };
 
@@ -92,7 +92,7 @@ static int rte_init(void)
 
     if (ORTE_PROC_IS_TOOL) {
         /* otherwise, if I am a tool proc, use that procedure */
-        if (ORTE_SUCCESS != (ret = orte_ess_base_tool_setup())) {
+        if (ORTE_SUCCESS != (ret = orte_ess_base_tool_setup(NULL))) {
             ORTE_ERROR_LOG(ret);
             error = "orte_ess_base_tool_setup";
             goto error;
@@ -101,19 +101,10 @@ static int rte_init(void)
 
     }
 
-    /* otherwise, I must be an application process - use
-     * the default procedure to finish my setup
-     */
-    if (ORTE_SUCCESS != (ret = orte_ess_base_app_setup(false))) {
-        ORTE_ERROR_LOG(ret);
-        error = "orte_ess_base_app_setup";
-        goto error;
-    }
-
     return ORTE_SUCCESS;
 
 error:
-    if (ORTE_ERR_SILENT != ret && !orte_report_silent_errors) {
+    if( ORTE_ERR_SILENT != ret || orte_report_silent_errors ) {
         orte_show_help("help-orte-runtime.txt",
                        "orte_init:startup:internal-failure",
                        true, error, ORTE_ERROR_NAME(ret), ret);
@@ -138,14 +129,6 @@ static int rte_finalize(void)
             ORTE_ERROR_LOG(ret);
         }
         return ret;
-    } else {
-        /* otherwise, I must be an application process
-         * use the default procedure to finish
-         */
-        if (ORTE_SUCCESS != (ret = orte_ess_base_app_finalize())) {
-            ORTE_ERROR_LOG(ret);
-            return ret;
-        }
     }
 
     return ORTE_SUCCESS;;
