@@ -5,16 +5,18 @@
  * Copyright (c) 2004-2006 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
- * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart, 
+ * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart,
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
- * Copyright (c) 2007-2011 Cisco Systems, Inc.  All rights reserved.
+ * Copyright (c) 2007-2015 Cisco Systems, Inc.  All rights reserved.
  * Copyright (c) 2009      Oak Ridge National Labs.  All rights reserved.
+ * Copyright (c) 2015-2016 Research Organization for Information Science
+ *                         and Technology (RIST). All rights reserved.
  * $COPYRIGHT$
- * 
+ *
  * Additional copyrights may follow
- * 
+ *
  * $HEADER$
  */
 
@@ -33,6 +35,7 @@
 #endif
 
 #include MCA_timer_IMPLEMENTATION_HEADER
+#include "opal/include/opal/version.h"
 #include "opal/mca/installdirs/installdirs.h"
 #include "opal/class/opal_value_array.h"
 #include "opal/class/opal_pointer_array.h"
@@ -78,9 +81,9 @@ void orte_info_do_params(bool want_all_in, bool want_internal)
     bool found;
     int i;
     bool want_all = false;
-    
+
     orte_info_components_open();
-    
+
     if (want_all_in) {
         want_all = true;
     } else {
@@ -96,8 +99,8 @@ void orte_info_do_params(bool want_all_in, bool want_internal)
             }
         }
     }
-    
-    /* Show the params */    
+
+    /* Show the params */
     if (want_all) {
         for (i = 0; i < mca_types.size; ++i) {
             if (NULL == (type = (char *)opal_pointer_array_get_item(&mca_types, i))) {
@@ -109,7 +112,7 @@ void orte_info_do_params(bool want_all_in, bool want_internal)
         for (i = 0; i < count; ++i) {
             type = opal_cmd_line_get_param(orte_info_cmd_line, "param", (int)i, 0);
             component = opal_cmd_line_get_param(orte_info_cmd_line, "param", (int)i, 1);
-            
+
             for (found = false, i = 0; i < mca_types.size; ++i) {
                 if (NULL == (str = (char *)opal_pointer_array_get_item(&mca_types, i))) {
                     continue;
@@ -119,14 +122,14 @@ void orte_info_do_params(bool want_all_in, bool want_internal)
                     break;
                 }
             }
-            
+
             if (!found) {
                 char *usage = opal_cmd_line_get_usage_msg(orte_info_cmd_line);
                 orte_show_help("help-orte-info.txt", "not-found", true, type);
                 free(usage);
                 exit(1);
             }
-            
+
             orte_info_show_mca_params(type, component, want_internal);
         }
     }
@@ -182,7 +185,7 @@ static void orte_info_show_mca_group_params(const mca_base_var_group_t *group, b
     }
 }
 
-void orte_info_show_mca_params(const char *type, const char *component, 
+void orte_info_show_mca_params(const char *type, const char *component,
                                bool want_internal)
 {
     const mca_base_var_group_t *group;
@@ -212,7 +215,7 @@ void orte_info_do_path(bool want_all, opal_cmd_line_t *cmd_line)
 {
     int i, count;
     char *scope;
-    
+
     /* Check bozo case */
     count = opal_cmd_line_get_ninsts(cmd_line, "path");
     for (i = 0; i < count; ++i) {
@@ -222,7 +225,7 @@ void orte_info_do_path(bool want_all, opal_cmd_line_t *cmd_line)
             break;
         }
     }
-    
+
     if (want_all) {
         orte_info_show_path(orte_info_path_prefix, opal_install_dirs.prefix);
         orte_info_show_path(orte_info_path_exec_prefix, opal_install_dirs.exec_prefix);
@@ -231,7 +234,7 @@ void orte_info_do_path(bool want_all, opal_cmd_line_t *cmd_line)
         orte_info_show_path(orte_info_path_libdir, opal_install_dirs.libdir);
         orte_info_show_path(orte_info_path_incdir, opal_install_dirs.includedir);
         orte_info_show_path(orte_info_path_mandir, opal_install_dirs.mandir);
-        orte_info_show_path(orte_info_path_pkglibdir, opal_install_dirs.ompilibdir);
+        orte_info_show_path(orte_info_path_pkglibdir, opal_install_dirs.opallibdir);
         orte_info_show_path(orte_info_path_libexecdir, opal_install_dirs.libexecdir);
         orte_info_show_path(orte_info_path_datarootdir, opal_install_dirs.datarootdir);
         orte_info_show_path(orte_info_path_datadir, opal_install_dirs.datadir);
@@ -239,14 +242,14 @@ void orte_info_do_path(bool want_all, opal_cmd_line_t *cmd_line)
         orte_info_show_path(orte_info_path_sharedstatedir, opal_install_dirs.sharedstatedir);
         orte_info_show_path(orte_info_path_localstatedir, opal_install_dirs.localstatedir);
         orte_info_show_path(orte_info_path_infodir, opal_install_dirs.infodir);
-        orte_info_show_path(orte_info_path_pkgdatadir, opal_install_dirs.ompidatadir);
-        orte_info_show_path(orte_info_path_pkglibdir, opal_install_dirs.ompilibdir);
-        orte_info_show_path(orte_info_path_pkgincludedir, opal_install_dirs.ompiincludedir);
+        orte_info_show_path(orte_info_path_pkgdatadir, opal_install_dirs.opaldatadir);
+        orte_info_show_path(orte_info_path_pkglibdir, opal_install_dirs.opallibdir);
+        orte_info_show_path(orte_info_path_pkgincludedir, opal_install_dirs.opalincludedir);
     } else {
         count = opal_cmd_line_get_ninsts(cmd_line, "path");
         for (i = 0; i < count; ++i) {
             scope = opal_cmd_line_get_param(cmd_line, "path", i, 0);
-            
+
             if (0 == strcmp(orte_info_path_prefix, scope)) {
                 orte_info_show_path(orte_info_path_prefix, opal_install_dirs.prefix);
             } else if (0 == strcmp(orte_info_path_bindir, scope)) {
@@ -258,7 +261,7 @@ void orte_info_do_path(bool want_all, opal_cmd_line_t *cmd_line)
             } else if (0 == strcmp(orte_info_path_mandir, scope)) {
                 orte_info_show_path(orte_info_path_mandir, opal_install_dirs.mandir);
             } else if (0 == strcmp(orte_info_path_pkglibdir, scope)) {
-                orte_info_show_path(orte_info_path_pkglibdir, opal_install_dirs.ompilibdir);
+                orte_info_show_path(orte_info_path_pkglibdir, opal_install_dirs.opallibdir);
             } else if (0 == strcmp(orte_info_path_sysconfdir, scope)) {
                 orte_info_show_path(orte_info_path_sysconfdir, opal_install_dirs.sysconfdir);
             } else if (0 == strcmp(orte_info_path_exec_prefix, scope)) {
@@ -278,9 +281,9 @@ void orte_info_do_path(bool want_all, opal_cmd_line_t *cmd_line)
             } else if (0 == strcmp(orte_info_path_infodir, scope)) {
                 orte_info_show_path(orte_info_path_infodir, opal_install_dirs.infodir);
             } else if (0 == strcmp(orte_info_path_pkgdatadir, scope)) {
-                orte_info_show_path(orte_info_path_pkgdatadir, opal_install_dirs.ompidatadir);
+                orte_info_show_path(orte_info_path_pkgdatadir, opal_install_dirs.opaldatadir);
             } else if (0 == strcmp(orte_info_path_pkgincludedir, scope)) {
-                orte_info_show_path(orte_info_path_pkgincludedir, opal_install_dirs.ompiincludedir);
+                orte_info_show_path(orte_info_path_pkgincludedir, opal_install_dirs.opalincludedir);
             } else {
                 char *usage = opal_cmd_line_get_usage_msg(cmd_line);
                 orte_show_help("help-orte-info.txt", "usage", true, usage);
@@ -295,10 +298,10 @@ void orte_info_do_path(bool want_all, opal_cmd_line_t *cmd_line)
 void orte_info_show_path(const char *type, const char *value)
 {
     char *pretty, *path;
-    
+
     pretty = strdup(type);
     pretty[0] = toupper(pretty[0]);
-    
+
     asprintf(&path, "path:%s", type);
     orte_info_out(pretty, path, value);
     free(pretty);
@@ -325,8 +328,8 @@ void orte_info_do_hostname()
  *				  FALSE -> display selected options
  *
  * This function displays all the options with which the current
- * installation of orte was configured. There are many options here 
- * that are carried forward from OMPI-7 and are not mca parameters 
+ * installation of orte was configured. There are many options here
+ * that are carried forward from OMPI-7 and are not mca parameters
  * in OMPI-10. I have to dig through the invalid options and replace
  * them with OMPI-10 options.
  */
@@ -337,39 +340,36 @@ void orte_info_do_config(bool want_all)
     char *memdebug;
     char *debug;
     char *threads;
-    char *want_libltdl;
+    char *have_dl;
     char *orterun_prefix_by_default;
     char *wtime_support;
     char *symbol_visibility;
     char *ft_support;
-    
+
     /* setup the strings that don't require allocations*/
     heterogeneous = OPAL_ENABLE_HETEROGENEOUS_SUPPORT ? "yes" : "no";
     memprofile = OPAL_ENABLE_MEM_PROFILE ? "yes" : "no";
     memdebug = OPAL_ENABLE_MEM_DEBUG ? "yes" : "no";
     debug = OPAL_ENABLE_DEBUG ? "yes" : "no";
-    want_libltdl = OPAL_WANT_LIBLTDL ? "yes" : "no";
+    have_dl = OPAL_HAVE_DL_SUPPORT ? "yes" : "no";
     orterun_prefix_by_default = ORTE_WANT_ORTERUN_PREFIX_BY_DEFAULT ? "yes" : "no";
     wtime_support = OPAL_TIMER_USEC_NATIVE ? "native" : "gettimeofday";
     symbol_visibility = OPAL_C_HAVE_VISIBILITY ? "yes" : "no";
-    
-    /* setup strings that require allocation */    
-    if (OPAL_HAVE_POSIX_THREADS) {        /* should just test OPAL_HAVE_THREADS */
-        asprintf(&threads, "%s (OPAL: %s, ORTE progress: yes, Event lib: yes)", 
-                 (OPAL_HAVE_POSIX_THREADS ? "posix" : "type unknown"), /* "type unknown" can presumably never happen */
-                 OPAL_ENABLE_MULTI_THREADS ? "yes" : "no");
-    } else {
-        threads = strdup("no");
-    }
-    
-    asprintf(&ft_support, "%s (checkpoint thread: %s)", 
+
+    /* setup strings that require allocation */
+    asprintf(&threads, "%s (OPAL: %s, ORTE progress: yes, Event lib: yes)",
+             "posix",
+             OPAL_ENABLE_MULTI_THREADS ? "yes" : "no");
+
+    asprintf(&ft_support, "%s (checkpoint thread: %s)",
              OPAL_ENABLE_FT ? "yes" : "no", OPAL_ENABLE_FT_THREAD ? "yes" : "no");;
-    
+
     /* output values */
     orte_info_out("Configured by", "config:user", OPAL_CONFIGURE_USER);
     orte_info_out("Configured on", "config:timestamp", OPAL_CONFIGURE_DATE);
     orte_info_out("Configure host", "config:host", OPAL_CONFIGURE_HOST);
-    
+    orte_info_out("Configure command line", "config:cli", OPAL_CONFIGURE_CLI);
+
     orte_info_out("Built by", "build:user", OMPI_BUILD_USER);
     orte_info_out("Built on", "build:timestamp", OMPI_BUILD_DATE);
     orte_info_out("Built host", "build:host", OMPI_BUILD_HOST);
@@ -378,7 +378,7 @@ void orte_info_do_config(bool want_all)
     orte_info_out("C compiler absolute", "compiler:c:absolute", OPAL_CC_ABSOLUTE);
     orte_info_out("C compiler family name", "compiler:c:familyname", _STRINGIFY(OPAL_BUILD_PLATFORM_COMPILER_FAMILYNAME));
     orte_info_out("C compiler version", "compiler:c:version", _STRINGIFY(OPAL_BUILD_PLATFORM_COMPILER_VERSION_STR));
-    
+
     if (want_all) {
         orte_info_out_int("C char size", "compiler:c:sizeof:char", sizeof(char));
         /* JMS: should be fixed in MPI-2.2 to differentiate between C
@@ -401,36 +401,36 @@ void orte_info_do_config(bool want_all)
         orte_info_out_int("C float align", "compiler:c:align:float", OPAL_ALIGNMENT_FLOAT);
         orte_info_out_int("C double align", "compiler:c:align:double", OPAL_ALIGNMENT_DOUBLE);
     }
-    
+
     orte_info_out("Thread support", "option:threads", threads);
 
     if (want_all) {
-        
-        
+
+
         orte_info_out("Build CFLAGS", "option:build:cflags", OMPI_BUILD_CFLAGS);
         orte_info_out("Build LDFLAGS", "option:build:ldflags", OMPI_BUILD_LDFLAGS);
         orte_info_out("Build LIBS", "option:build:libs", OMPI_BUILD_LIBS);
-        
-        orte_info_out("Wrapper extra CFLAGS", "option:wrapper:extra_cflags", 
+
+        orte_info_out("Wrapper extra CFLAGS", "option:wrapper:extra_cflags",
                       WRAPPER_EXTRA_CFLAGS);
-        orte_info_out("Wrapper extra LDFLAGS", "option:wrapper:extra_ldflags", 
+        orte_info_out("Wrapper extra LDFLAGS", "option:wrapper:extra_ldflags",
                       WRAPPER_EXTRA_LDFLAGS);
         orte_info_out("Wrapper extra LIBS", "option:wrapper:extra_libs",
                       WRAPPER_EXTRA_LIBS);
     }
     free(threads);
-    
+
     orte_info_out("Internal debug support", "option:debug", debug);
     orte_info_out("Memory profiling support", "option:mem-profile", memprofile);
     orte_info_out("Memory debugging support", "option:mem-debug", memdebug);
-    orte_info_out("libltdl support", "option:dlopen", want_libltdl);
+    orte_info_out("dl support", "option:dlopen", have_dl);
     orte_info_out("Heterogeneous support", "options:heterogeneous", heterogeneous);
-    orte_info_out("orterun default --prefix", "orterun:prefix_by_default", 
+    orte_info_out("orterun default --prefix", "orterun:prefix_by_default",
                   orterun_prefix_by_default);
     orte_info_out("MPI_WTIME support", "options:mpi-wtime", wtime_support);
     orte_info_out("Symbol vis. support", "options:visibility", symbol_visibility);
-    
+
     orte_info_out("FT Checkpoint support", "options:ft_support", ft_support);
     free(ft_support);
-    
+
 }
