@@ -2,17 +2,17 @@
  * Copyright (c) 2013      Mellanox Technologies, Inc.
  *                         All rights reserved.
  * $COPYRIGHT$
- * 
+ *
  * Additional copyrights may follow
- * 
+ *
  * $HEADER$
  */
 /**
- *  @file 
+ *  @file
  */
 
-#ifndef MCA_SPML_YODA_COMPONENT_H
-#define MCA_SPML_YODA_COMPONENT_H
+#ifndef MCA_SPML_IKRIT_COMPONENT_H
+#define MCA_SPML_IKRIT_COMPONENT_H
 
 BEGIN_C_DECLS
 
